-- frontier_silo.lua
-- Jan 2018
-- My take on frontier silos for my Oarc scenario

require("config")
require("DE333/DE333_config")
require("lib/oarc_utils")

local market_items = require "DE333.market.market_items"
local math_random = math.random

--------------------------------------------------------------------------------
-- Frontier style rocket silo stuff
--------------------------------------------------------------------------------

-- This creates a random silo position, stored to global.siloPosition
-- It uses the config setting global.ocfg.frontier_silo_distance and spawns the
-- silo somewhere on a circle edge with radius using that distance.
function SetRandomSiloPosition(num_silos)
    if (global.siloPosition == nil) then
        global.siloPosition = {}
    end

    random_angle_offset = math.random(0, math.pi * 2)

    for i=1,num_silos do
        theta = ((math.pi * 2) / num_silos);
        angle = (theta * i) + random_angle_offset;

        tx = (global.ocfg.frontier_silo_distance*CHUNK_SIZE * math.cos(angle))
        ty = (global.ocfg.frontier_silo_distance*CHUNK_SIZE * math.sin(angle))

        table.insert(global.siloPosition, {x=math.floor(tx), y=math.floor(ty)})

        log("Silo position: " .. tx .. ", " .. ty .. ", " .. angle)
    end
end

-- Sets the global.siloPosition var to the set in the config file
function SetFixedSiloPosition(pos)
    table.insert(global.siloPosition, pos)
end

-- Create a rocket silo at the specified positionmmmm
-- Also makes sure tiles and entities are cleared if required.
local function CreateRocketSilo(surface, siloPosition, force)

    -- Delete any entities beneath the silo?
    for _, entity in pairs(surface.find_entities_filtered{area = {{siloPosition.x-5,
                                                                    siloPosition.y-6},
                                                                    {siloPosition.x+6,
                                                                    siloPosition.y+6}}}) do
        entity.destroy()
    end

    -- Remove nearby enemies again
    for _, entity in pairs(surface.find_entities_filtered{area = {{siloPosition.x-(CHUNK_SIZE*4),
                                                                    siloPosition.y-(CHUNK_SIZE*4)},
                                                                    {siloPosition.x+(CHUNK_SIZE*4),
                                                                    siloPosition.y+(CHUNK_SIZE*4)}}, force = "enemy"}) do
        entity.destroy()
    end

    -- Set tiles below the silo
    tiles = {}
    i = 1
    for dx = -5,5 do
        for dy = -5,5 do
            tiles[i] = {name = "concrete", position = {siloPosition.x+dx, siloPosition.y+dy}}
            i=i+1
        end
    end
    surface.set_tiles(tiles, true)

    -- Create indestructible silo and assign to a force
    local silo = surface.create_entity{name = "rocket-silo", position = {siloPosition.x+0.5, siloPosition.y}, force = force}
    silo.destructible = false
    silo.minable = false

    -- TAG it on the main force at least.
    game.forces[global.ocfg.main_force].add_chart_tag(game.surfaces[GAME_SURFACE_NAME],
                                            {position=siloPosition, text="Rocket Silo",
                                                icon={type="item",name="rocket-silo"}})

    -- Make silo safe from being removed by regrowth
    -- if global.ocfg.enable_regrowth then
        OarcRegrowthOffLimits(siloPosition, 5)
    -- end


    if ENABLE_SILO_BEACONS then
        PhilipsBeacons(surface, siloPosition, game.forces[global.ocfg.main_force])
    end
    if ENABLE_SILO_RADAR then
        PhilipsRadar(surface, siloPosition, game.forces[global.ocfg.main_force])
    end
	
	if ENABLE_MARKET then
       Market(surface, siloPosition, game.forces[global.ocfg.main_force])
    end
	
	if ENABLE_MARKET_SHIP then
       MarketShip(surface, siloPosition, game.forces[MAIN_FORCE])
	   silowreckage(surface, siloPosition, game.forces[MAIN_FORCE])
    end
	
	if ENABLE_SILO_BITER then
       silobiter(surface, siloPosition, game.forces[MAIN_FORCE])
    end
        
end

-- Generates all rocket silos, should be called after the areas are generated
-- Includes a crop circle
function GenerateAllSilos(surface)                       
    
    -- Create each silo in the list
    for idx,siloPos in pairs(global.siloPosition) do
        CreateRocketSilo(surface, siloPos, global.ocfg.main_force)
    end
end

-- Generate clean land and trees around silo area on chunk generate event
function GenerateRocketSiloChunk(event)

    -- Silo generation can take awhile depending on the number of silos.
    if (game.tick < global.ocfg.frontier_silo_count*10*TICKS_PER_SECOND) then
        local surface = event.surface
        local chunkArea = event.area

        local chunkAreaCenter = {x=chunkArea.left_top.x+(CHUNK_SIZE/2),
                                 y=chunkArea.left_top.y+(CHUNK_SIZE/2)}
        
        for idx,siloPos in pairs(global.siloPosition) do
            local safeArea = {left_top=
                                {x=siloPos.x-(CHUNK_SIZE*4),
                                 y=siloPos.y-(CHUNK_SIZE*4)},
                              right_bottom=
                                {x=siloPos.x+(CHUNK_SIZE*4),
                                 y=siloPos.y+(CHUNK_SIZE*4)}}
                                     
            if CheckIfInArea(chunkAreaCenter,safeArea) then
                -- Remove trees/resources inside the spawn area
                RemoveInCircle(surface, chunkArea, "tree", siloPos, global.ocfg.spawn_config.gen_settings.land_area_tiles+5)
                RemoveInCircle(surface, chunkArea, "resource", siloPos, global.ocfg.spawn_config.gen_settings.land_area_tiles+5)
                RemoveInCircle(surface, chunkArea, "cliff", siloPos, global.ocfg.spawn_config.gen_settings.land_area_tiles+5)
                RemoveDecorationsArea(surface, chunkArea)

                -- Create rocket silo
                CreateCropOctagon(surface, siloPos, chunkArea, CHUNK_SIZE)
            end
        end
    end
end

-- Generate chunks where we plan to place the rocket silos.
function GenerateRocketSiloAreas(surface)
    for idx,siloPos in pairs(global.siloPosition) do
        surface.request_to_generate_chunks({siloPos.x, siloPos.y}, 3)
    end
	if (global.ocfg.frontier_silo_vision) then
        ChartRocketSiloAreas(surface, game.forces[global.ocfg.main_force])
    end
end

-- Chart chunks where we plan to place the rocket silos.
function ChartRocketSiloAreas(surface, force)
    for idx,siloPos in pairs(global.siloPosition) do
        force.chart(surface, {{siloPos.x-(CHUNK_SIZE*2),
                                siloPos.y-(CHUNK_SIZE*2)},
                                {siloPos.x+(CHUNK_SIZE*2),
                                siloPos.y+(CHUNK_SIZE*2)}})
    end
end

global.oarc_silos_generated = false
function DelayedSiloCreationOnTick(surface)

    -- Delay the creation of the silos so we place them on already generated lands.
    if (not global.oarc_silos_generated and (game.tick >= global.ocfg.frontier_silo_count*10*TICKS_PER_SECOND)) then
        log("Frontier silos generated!")
        global.oarc_silos_generated = true
        GenerateAllSilos(surface)
    end

end 


function PhilipsBeacons(surface, siloPos, force)

    -- Add Beacons
    -- x = right, left; y = up, down
    -- top 1 left 1
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- top 2
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-5, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- top 3
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-2, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- top 4
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+2, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- top 5
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+5, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- top 6 right 1
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y-8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 2
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y-5}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 3
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y-2}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 4
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y+2}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 5
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y+5}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 6 bottom 1
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-8, siloPos.y+8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- left 7 bottom 2
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x-5, siloPos.y+8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 2
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y-5}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 3
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y-2}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 4
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y+2}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 5
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y+5}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 6 bottom 3
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+5, siloPos.y+8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- right 7 bottom 4
    local beacon = surface.create_entity{name = "beacon", position = {siloPos.x+8, siloPos.y+8}, force = force}
    beacon.destructible = false
    beacon.minable = false
    -- substations
    -- top left
    local substation = surface.create_entity{name = "substation", position = {siloPos.x-5, siloPos.y-5}, force = force}
    substation.destructible = false
    substation.minable = false
    -- top right
    local substation = surface.create_entity{name = "substation", position = {siloPos.x+6, siloPos.y-5}, force = force}
    substation.destructible = false
    substation.minable = false
    -- bottom left
    local substation = surface.create_entity{name = "substation", position = {siloPos.x-5, siloPos.y+6}, force = force}
    substation.destructible = false
    substation.minable = false
    -- bottom right
    local substation = surface.create_entity{name = "substation", position = {siloPos.x+6, siloPos.y+6}, force = force}
    substation.destructible = false
    substation.minable = false

    -- end adding beacons
end

function PhilipsRadar(surface, siloPos, force)
    
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-43, siloPos.y+3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-43, siloPos.y-3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-40, siloPos.y-6}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-37, siloPos.y-6}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-34, siloPos.y-6}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-34, siloPos.y-3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-34, siloPos.y}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-34, siloPos.y+3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-43, siloPos.y-6}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-40, siloPos.y+3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "solar-panel", position = {siloPos.x-37, siloPos.y+3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "radar", position = {siloPos.x-43, siloPos.y}, force = force}
    radar.destructible = false
    local substation = surface.create_entity{name = "substation", position = {siloPos.x-38, siloPos.y-1}, force = force}
    substation.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-40, siloPos.y-1}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-40, siloPos.y-3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-40, siloPos.y+1}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-38, siloPos.y-3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-38, siloPos.y+1}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-36, siloPos.y-1}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-36, siloPos.y-3}, force = force}
    radar.destructible = false
    local radar = surface.create_entity{name = "accumulator", position = {siloPos.x-36, siloPos.y+1}, force = force}
    radar.destructible = false
end

function Market(surface, siloPos, force)

	local market = surface.create_entity{name = "market", position = {siloPos.x+0, siloPos.y-14}, force = force}
    market.destructible = false
	
	for _, item in pairs(market_items.spawn) do
			market.add_market_item(item)
	end
end

function MarketShip(surface, siloPos, force)

local chest_raffle_SILO = {}
	local chest_loot_SILO = {			
		--{{name = "submachine-gun", count = math_random(1,3)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},		
		--{{name = "slowdown-capsule", count = math_random(16,32)}, weight = 1, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "poison-capsule", count = math_random(16,32)}, weight = 3, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "uranium-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.6, evolution_max = 1},
		--{{name = "cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 0.7},
		--{{name = "explosive-uranium-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.6, evolution_max = 1},
		--{{name = "explosive-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 0.8},
		--{{name = "shotgun", count = 1}, weight = 2, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "shotgun-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "combat-shotgun", count = 1}, weight = 3, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "piercing-shotgun-shell", count = math_random(16,32)}, weight = 10, evolution_min = 0.2, evolution_max = 1},
		--{{name = "flamethrower", count = 1}, weight = 3, evolution_min = 0.3, evolution_max = 0.6},
		--{{name = "flamethrower-ammo", count = math_random(16,32)}, weight = 5, evolution_min = 0.3, evolution_max = 1},
		--{{name = "rocket-launcher", count = 1}, weight = 3, evolution_min = 0.2, evolution_max = 0.6},
		--{{name = "rocket", count = math_random(16,32)}, weight = 5, evolution_min = 0.2, evolution_max = 0.7},		
		--{{name = "explosive-rocket", count = math_random(16,32)}, weight = 5, evolution_min = 0.3, evolution_max = 1},
		--{{name = "land-mine", count = math_random(16,32)}, weight = 5, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "grenade", count = math_random(16,32)}, weight = 5, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "cluster-grenade", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 1},
		--{{name = "firearm-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0, evolution_max = 0.3},
		--{{name = "piercing-rounds-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0.1, evolution_max = 0.8},
		{{name = "uranium-rounds-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0.5, evolution_max = 1},
		--{{name = "railgun", count = 1}, weight = 1, evolution_min = 0.2, evolution_max = 1},
		--{{name = "railgun-dart", count = math_random(16,32)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "defender-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.0, evolution_max = 0.7},
		--{{name = "distractor-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 1},
		--{{name = "destroyer-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.3, evolution_max = 1},
		--{{name = "atomic-bomb", count = math_random(8,16)}, weight = 1, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "light-armor", count = 1}, weight = 3, evolution_min = 0, evolution_max = 0.1},		
		--{{name = "heavy-armor", count = 1}, weight = 3, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "modular-armor", count = 1}, weight = 2, evolution_min = 0.2, evolution_max = 0.6},
		{{name = "power-armor", count = 1}, weight = 2, evolution_min = 0.4, evolution_max = 1},
		--{{name = "power-armor-mk2", count = 1}, weight = 1, evolution_min = 0.9, evolution_max = 1},
		{{name = "battery-equipment", count = 1}, weight = 2, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "battery-mk2-equipment", count = 1}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		{{name = "belt-immunity-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
		{{name = "solar-panel-equipment", count = math_random(1,4)}, weight = 5, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "discharge-defense-equipment", count = 1}, weight = 1, evolution_min = 0.5, evolution_max = 0.8},
		{{name = "energy-shield-equipment", count = math_random(1,2)}, weight = 2, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "energy-shield-mk2-equipment", count = 1}, weight = 2, evolution_min = 0.7, evolution_max = 1},
		{{name = "exoskeleton-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
		{{name = "fusion-reactor-equipment", count = 1}, weight = 1, evolution_min = 0.5, evolution_max = 1},
		{{name = "night-vision-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "personal-laser-defense-equipment", count = 1}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		{{name = "exoskeleton-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
						
		
		--{{name = "iron-gear-wheel", count = math_random(80,100)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "copper-cable", count = math_random(100,200)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "electric-engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.4, evolution_max = 0.8},
		--{{name = "battery", count = math_random(50,150)}, weight = 2, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "advanced-circuit", count = math_random(50,150)}, weight = 3, evolution_min = 0.4, evolution_max = 1},
		--{{name = "electronic-circuit", count = math_random(50,150)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		--{{name = "processing-unit", count = math_random(50,150)}, weight = 3, evolution_min = 0.7, evolution_max = 1},
		--{{name = "explosives", count = math_random(40,50)}, weight = 10, evolution_min = 0.0, evolution_max = 1},
		--{{name = "lubricant-barrel", count = math_random(4,10)}, weight = 1, evolution_min = 0.3, evolution_max = 0.5},
		--{{name = "rocket-fuel", count = math_random(4,10)}, weight = 2, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "computer", count = 1}, weight = 2, evolution_min = 0, evolution_max = 1},
		--{{name = "steel-plate", count = math_random(25,75)}, weight = 2, evolution_min = 0.1, evolution_max = 0.3},
		{{name = "nuclear-fuel", count = 1}, weight = 2, evolution_min = 0.7, evolution_max = 1},
				
		--{{name = "burner-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},
		--{{name = "inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		--{{name = "long-handed-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},		
		--{{name = "fast-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.1, evolution_max = 1},
		--{{name = "filter-inserter", count = math_random(8,16)}, weight = 1, evolution_min = 0.2, evolution_max = 1},		
		--{{name = "stack-filter-inserter", count = math_random(4,8)}, weight = 1, evolution_min = 0.4, evolution_max = 1},
		--{{name = "stack-inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},				
		--{{name = "small-electric-pole", count = math_random(16,24)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "medium-electric-pole", count = math_random(8,16)}, weight = 3, evolution_min = 0.2, evolution_max = 1},
		--{{name = "big-electric-pole", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},
		{{name = "substation", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "wooden-chest", count = math_random(16,24)}, weight = 3, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "iron-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.1, evolution_max = 0.4},
		--{{name = "steel-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "small-lamp", count = math_random(16,32)}, weight = 3, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "rail", count = math_random(25,75)}, weight = 3, evolution_min = 0.1, evolution_max = 0.6},
		--{{name = "assembling-machine-1", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "assembling-machine-2", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.8},
		{{name = "assembling-machine-3", count = math_random(1,2)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "accumulator", count = math_random(4,8)}, weight = 3, evolution_min = 0.4, evolution_max = 1},
		--{{name = "offshore-pump", count = math_random(1,3)}, weight = 2, evolution_min = 0.0, evolution_max = 0.1},
		{{name = "beacon", count = math_random(1,2)}, weight = 3, evolution_min = 0.7, evolution_max = 1},
		--{{name = "boiler", count = math_random(4,8)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "steam-engine", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "steam-turbine", count = math_random(1,2)}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		--{{name = "nuclear-reactor", count = 1}, weight = 1, evolution_min = 0.6, evolution_max = 1},
		{{name = "centrifuge", count = math_random(1,2)}, weight = 1, evolution_min = 0.6, evolution_max = 1},
		--{{name = "heat-pipe", count = math_random(4,8)}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		--{{name = "heat-exchanger", count = math_random(2,4)}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		--{{name = "arithmetic-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "constant-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "decider-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "power-switch", count = math_random(1,2)}, weight = 1, evolution_min = 0.1, evolution_max = 1},		
		--{{name = "programmable-speaker", count = math_random(4,8)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "green-wire", count = math_random(50,99)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "red-wire", count = math_random(50,99)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "chemical-plant", count = math_random(1,3)}, weight = 3, evolution_min = 0.3, evolution_max = 1},
		--{{name = "burner-mining-drill", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "electric-mining-drill", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.6},		
		{{name = "express-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "express-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.5, evolution_max = 1},		
		{{name = "express-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "fast-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "fast-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "fast-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.3},
		--{{name = "transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		--{{name = "underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		--{{name = "splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0, evolution_max = 0.3},		
		--{{name = "oil-refinery", count = math_random(2,4)}, weight = 2, evolution_min = 0.3, evolution_max = 1},
		--{{name = "pipe", count = math_random(30,50)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "pipe-to-ground", count = math_random(4,8)}, weight = 1, evolution_min = 0.2, evolution_max = 0.5},
		--{{name = "pumpjack", count = math_random(1,3)}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "pump", count = math_random(1,2)}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "solar-panel", count = math_random(8,16)}, weight = 3, evolution_min = 0.4, evolution_max = 0.9},
		{{name = "electric-furnace", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "steel-furnace", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "stone-furnace", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},		
		--{{name = "radar", count = math_random(1,2)}, weight = 1, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "rail-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},
		--{{name = "rail-chain-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},		
		--{{name = "stone-wall", count = math_random(25,75)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "gate", count = math_random(4,8)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "storage-tank", count = math_random(1,4)}, weight = 3, evolution_min = 0.3, evolution_max = 0.6},
		--{{name = "train-stop", count = math_random(1,2)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "express-loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.5, evolution_max = 1},
		{{name = "fast-loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "lab", count = math_random(1,2)}, weight = 2, evolution_min = 0.0, evolution_max = 0.1},	
		{{name = "roboport", count = math_random(2,4)}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		--{{name = "flamethrower-turret", count = math_random(1,3)}, weight = 3, evolution_min = 0.5, evolution_max = 1},		
		--{{name = "laser-turret", count = math_random(4,8)}, weight = 3, evolution_min = 0.5, evolution_max = 1},	
		--{{name = "gun-turret", count = math_random(2,6)}, weight = 3, evolution_min = 0.2, evolution_max = 0.9},

		{{name = "coin", count = math_random(1000,2500)}, weight = 1, evolution_min = 0.1, evolution_max = 1}	
	}
	
	
	for _, t in pairs (chest_loot_SILO) do
		for x = 1, t.weight, 1 do
			table.insert(chest_raffle_SILO, t[1])
		end			
	end

    -- Create indestructible ship wreck
    local ship = surface.create_entity{name = "big-ship-wreck-3", position = {siloPos.x-11, siloPos.y+9}, force = force}
	ship.minable = false
	local i = ship.get_inventory(defines.inventory.chest)
	for x = 1, math_random(3,5), 1 do
		local loot = chest_raffle_SILO[math_random(1,#chest_raffle_SILO)]
		i.insert(loot)
	end
	
	local ship2 = surface.create_entity{name = "big-ship-wreck-1", position = {siloPos.x-5, siloPos.y+7}, force = force}
	ship2.minable = false
	local i = ship2.get_inventory(defines.inventory.chest)
	for x = 1, math_random(3,5), 1 do
		local loot = chest_raffle_SILO[math_random(1,#chest_raffle_SILO)]
		i.insert(loot)
	end
end

function silobiter(surface, siloPos)
	local spawner1 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+1, siloPos.y-8}}
	local spawner2 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x+7, siloPos.y-12}}
	local spawner3 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+10, siloPos.y-6}}
	local spawner4 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x+9, siloPos.y+4}}
	local spawner5 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x+8, siloPos.y+9}}
	local spawner6 = surface.create_entity{name = "biter-spawner", position = {siloPos.x-5, siloPos.y-10}}
	local spawner7 = surface.create_entity{name = "biter-spawner", position = {siloPos.x-12, siloPos.y-8}}
	local spawner8 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x-15, siloPos.y-2}}
	local spawner9 = surface.create_entity{name = "biter-spawner", position = {siloPos.x-11, siloPos.y+15}}
	local spawner10 = surface.create_entity{name = "biter-spawner", position = {siloPos.x-2, siloPos.y+13}}
	local spawner11 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+5, siloPos.y+17}}
	local spawner12 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+17, siloPos.y-0}}
	local spawner13 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x+19, siloPos.y+10}}
	local spawner14 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+21, siloPos.y-10}}
	local spawner15 = surface.create_entity{name = "biter-spawner", position = {siloPos.x-0, siloPos.y-18}}
	local spawner16 = surface.create_entity{name = "spitter-spawner", position = {siloPos.x-6, siloPos.y-19}}
	local spawner17 = surface.create_entity{name = "biter-spawner", position = {siloPos.x+7, siloPos.y-20}}
	local worm1 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x+7, siloPos.y-0}}
	local worm2 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x-8, siloPos.y-2}}
	local worm3 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x-8, siloPos.y+1}}
	local worm4 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x+3, siloPos.y+7}}
	local worm5 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x-0, siloPos.y+7}}
	local worm6 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x+18, siloPos.y-7}}
	local worm7 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x+18, siloPos.y-4}}
	local worm8 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x-4, siloPos.y-14}}
	local worm9 = surface.create_entity{name = "medium-worm-turret", position = {siloPos.x+10, siloPos.y-15}}
end

function silowreckage(surface, siloPos, force)
    local wreckage1 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-11, siloPos.y+12}, force = force}
    local wreckage2 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-10, siloPos.y+15}, force = force}
    local wreckage3 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-9, siloPos.y-13}, force = force}
	local wreckage4 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-12, siloPos.y-14}, force = force}
	local wreckage5 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+12, siloPos.y-10}, force = force}
    local wreckage6 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-10, siloPos.y-1}, force = force}
	local wreckage7 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-13, siloPos.y-12}, force = force}
	local wreckage8 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+13, siloPos.y+12}, force = force}
	local wreckage9 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-13, siloPos.y+12}, force = force}
	local wreckage10 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+14, siloPos.y-12}, force = force}
	local wreckage11 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+14, siloPos.y+13}, force = force}
	local wreckage12 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-14, siloPos.y+12}, force = force}
	local wreckage13 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+14, siloPos.y-17}, force = force}
	local wreckage14 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-14, siloPos.y-14}, force = force}
	local wreckage15 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-14, siloPos.y+12}, force = force}
	local wreckage16 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+15, siloPos.y-12}, force = force}
	local wreckage17 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+15, siloPos.y+13}, force = force}
	local wreckage18 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-14, siloPos.y+12}, force = force}
	local wreckage19 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+16, siloPos.y-17}, force = force}
	local wreckage20 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-17, siloPos.y-14}, force = force}
	local wreckage21 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-15, siloPos.y+12}, force = force}
	local wreckage22 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+15, siloPos.y-18}, force = force}
	local wreckage23 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-8, siloPos.y+26}, force = force}
	local wreckage24 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-7, siloPos.y+24}, force = force}
	local wreckage25 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+7, siloPos.y-16}, force = force}
	local wreckage26 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-7, siloPos.y-16}, force = force}
	local wreckage27 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-8, siloPos.y+4}, force = force}
	local wreckage28 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+6, siloPos.y-5}, force = force}
	local wreckage29 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-7, siloPos.y-4}, force = force}
	local wreckage30 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-8, siloPos.y+8}, force = force}
	local wreckage31 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-16, siloPos.y+13}, force = force}
	local wreckage32 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-13, siloPos.y+10}, force = force}
	local wreckage33 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-14, siloPos.y+8}, force = force}
	local wreckage34 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-16, siloPos.y+6}, force = force}
	local wreckage35 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-18, siloPos.y+3}, force = force}
	local wreckage36 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x-20, siloPos.y+1}, force = force}
	local wreckage37 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+13, siloPos.y+1}, force = force}
	local wreckage38 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+20, siloPos.y+4}, force = force}	
	local wreckage39 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+18, siloPos.y+6}, force = force}	
	local wreckage40 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+14, siloPos.y-6}, force = force}
	local wreckage41 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+11, siloPos.y-4}, force = force}
	local wreckage42 = surface.create_entity{name = "mineable-wreckage", position = {siloPos.x+10, siloPos.y-1}, force = force}	
end